(function (window) {
    if (window.ILCommon) {
        return;
    }
    ;
    var ILCommon = /** @class */ (function () {
        function ILCommon() {
        }
        ILCommon.prototype.onBeforeLoad = function (e) {
            if (/^(?!https?:)[\w-]+:/.test(e.url))
                return;
            var type = "OTHER";
            var eventName = "error";
            switch(e.target.localName) {
                case "frame":
                case "iframe":
                    type = "SUBDOCUMENT";
                    eventName = "load";
                    break;
                case "img":
                case "input":
                    type = "IMAGE";
                    break;
                case "video":
                case "audio":
                case "source":
                    type = "MEDIA";
                    break;
                case "object":
                case "embed":
                    type = "OBJECT";
                    break;
                case "script":
                    type = "SCRIPT";
                    break;
                case "link":
                    if (/\bstylesheet\b/i.test(e.target.rel))
                        type = "STYLESHEET";
                    break;
            }
            if (type === 'SUBDOCUMENT' ||
                type === 'IMAGE' ||
                type === 'MEDIA' ||
                type == 'OBJECT' ||
                type == 'STYLESHEET') {
                // Work around bugs.webkit.org/show_bug.cgi?id=65412
                // Allow the resource to load, but hide it afterwards.
                // Probably a normal site will never reach 250.
                this._request_blockCount++;
                if (this._request_blockCount > 250) {
                    console.log("ABORTING: blocked over 250 requests, probably an infinite loading loop");
                    this._request_blockCount = 0;
                }
                e.preventDefault();
                // Safari doesn't dispatch the expected events for elements that have been
                // prevented from loading by having their "beforeload" event cancelled.
                // That is a "load" event for blocked frames, and an "error" event for
                // other blocked elements. We need to dispatch those events manually here
                // to avoid breaking element collapsing and pages that rely on those events.
                setTimeout(function() {
                    var evt = document.createEvent("Event");
                    evt.initEvent(eventName);
                    e.target.dispatchEvent(evt);
                }, 0);
            }
        };
        ILCommon.prototype.checkLongPressEvent = function (x, y) {
            let e = document.elementFromPoint(x, y);
            if (e.tagName === 'IMG') {
                let img = e;
                return img.src&&img.src.length > 0;
            } else {
                let as = document.querySelectorAll('a:hover');
                if (as.length > 0) {
                    let a = as[0];
                    return a.href.length > 0;
                }
            }
            return e.tagName === 'IFRAME';
        };
        ILCommon.prototype.longPressEvent = function (x, y) {
            let data = {};
            let e = document.elementFromPoint(x, y);
            if (e.tagName === 'IMG') {
                let img = e;
                data.type = 'img';
                data.url = img.src;
                this.toDataUrl(img.src, function(imgData) {
                    if (imgData) {
                        data.imgData = imgData.replace(/^data:image\/(jpeg|gif|png|jpg);base64,/, "");
                    }
                    window.ILJSBridge.callNative('Common', 'longPressEvent', { 'data': data}, function (data) {}, function (data) {});
                });
            } else {
                let as = document.querySelectorAll('a:hover');
                if (as.length > 0) {
                    let a = as[0];
                    data.type = 'a';
                    data.url = a.href;
                    window.ILJSBridge.callNative('Common', 'longPressEvent', { 'data': data}, function (data) {}, function (data) {});
                }
            }
        };
        ILCommon.prototype.toDataUrl = function (url, callback) {
            callback(null);
        };
        ILCommon.prototype.readyState = function () {
            this.postReadyState('loading');
            if (document.readyState === 'interactive') {
                this.postReadyState('interactive');
            } else if (document.readyState === 'complete') {
                this.postReadyState('interactive');
                this.postReadyState('complete');
            }
            if (document.readyState !== 'complete') {
                document.addEventListener('readystatechange', event => {
                    this.postReadyState(event.target.readyState);
                });
            }
        };
        ILCommon.prototype.postReadyState = function (readyState) {
            let data = {}
            let isSub = self !== top;
            var url = isSub?document.referrer:document.URL;
            data['readyState'] = readyState;
            data['isSub'] = isSub;
            data['url'] = url;
            window.ILJSBridge.callNative('Common', 'readyState', {'data': data}, function (tdata) {}, function (tdata) {});
        };
        ILCommon.prototype.disablePWA = function () {
            if (navigator && navigator.serviceWorker) {
                let register = navigator.serviceWorker.register;
                Object.defineProperty(navigator.serviceWorker, 'register', {
                    configurable: true,
                    enumerable: true,
                    writable: true,
                    value: function () {
                        return null;
                    }
                })
            }
        };
        ILCommon.prototype.__defineGetter__('textSize', function () { return this._textSize; });
        ILCommon.prototype.__defineSetter__('textSize', function (textSize) {
            this._textSize = textSize;
            let bodys = document.getElementsByTagName('body');
            for (let body of bodys) {
                body.style.webkitTextSizeAdjust = textSize;
            }
        });
        ILCommon.prototype.__defineGetter__('touchCallout', function () { return this._touchCallout; });
        ILCommon.prototype.__defineSetter__('touchCallout', function (touchCallout) {
            this._touchCallout = touchCallout;
            let value = '';
            if (!touchCallout) {
                value = 'none'
            }
            document.documentElement.style.webkitTouchCallout = value;
        });
        return ILCommon;
    }());
    var ILCommonInstance = new ILCommon();
    var ILCommonConfig = /** @class */ (function () {
        function ILCommonConfig() {
        }
        ILCommonConfig.prototype.init = function () {
            window.ILCommon = ILCommonInstance;
            ILCommonInstance.readyState();
        };
        ILCommonConfig.prototype.__defineGetter__('imageBlock', function () { return this._imageBlock; });
        ILCommonConfig.prototype.__defineSetter__('imageBlock', function (imageBlock) {
            this._imageBlock = imageBlock;
            if (imageBlock) {
                document.addEventListener('beforeload', ILCommonInstance.onBeforeLoad, true);
            } else {
                document.removeEventListener('beforeload', ILCommonInstance.onBeforeLoad, true);
            }
        });
        return ILCommonConfig;
    }());
    window.ILCommonConfig = new ILCommonConfig();
    window.ILCommonConfig.init();
})(window);
