(function (window) {
    if (window.ILTheme) {
        return;
    }
    ;
    var ILTheme = /** @class */ (function () {
        function ILTheme() {
            this.alphaLimit = 0.15;
            this.style = 'common';
            this.css = {};
            this.observer = null;
            this.darkMode = false;
            this.transparentMode = false;
            this.injectDelayMode = false;
            this.delayMode = true;
            this.delayEcho = 15;
            this.delayExpandTime = 20;
            this.targetNode = document.documentElement;
            this.config = { attributes: true, childList: true, subtree: true, attributeFilter: ['class', 'style']};
        }
        ILTheme.prototype.injectStyles = function () {
            let root = this.targetNode;
            if (root) {
                // let d = '';
                // d += 'background: url("http://b-ssl.duitang.com/uploads/blog/201510/17/20151017203227_JZiAd.jpeg") !important;';
                // this.setStyle(root, d);

                let elements = root.getElementsByTagName("*");
                let elementsInView = new Array();
                let delayElements = new Array();
                elementsInView.push(root);
                for (let t of elements) {
                    if (!this.injectDelayMode || this.isElementInViewport(t) || this.nodeNameEqual(t, 'BODY')) {
                        elementsInView.push(t);
                    } else {
                        delayElements.push(t);
                    }
                }
                let len = parseInt(delayElements.length/this.delayEcho);
                len = len<=0?1:len;
                let interval = 0;
                for(let i = 0; i < delayElements.length; i += len) {
                    let subDelayElements = delayElements.slice(i, i+len);
                    setTimeout(function() {
                        for (let t of subDelayElements) {
                            this.processStyles(t);
                        }
                    }.bind(this), interval);
                    interval+=this.delayExpandTime;
                }
                for (let t of elementsInView) {
                    this.processStyles(t);
                }
            }
        };
        ILTheme.prototype.ignoreStyles = function (t) {
            if (!t.getAttribute) {
                return false;
            }
            let ignoreThemeList = (t.getAttribute('ilthemeig')||'').split(',');
            return ignoreThemeList.indexOf(this.style) >= 0?true:false;
        };
        ILTheme.prototype.processStyles = function (t) {
            let d = "";
            let tmp = t;
            while (tmp) {
                if (this.ignoreStyles(tmp)) {
                    return;
                }
                tmp = tmp.parentNode;
            }
            if (this.nodeNameEqual(t, 'SCRIPT') ||
                this.nodeNameEqual(t, 'STYLE') ||
                this.nodeNameEqual(t, 'HEAD') ||
                this.nodeNameEqual(t, 'META') ||
                this.nodeNameEqual(t, 'LINK')) return;
            if (this.nodeNameEqual(t, 'IFRAME')) {
                let readyState = t.getAttribute('il_readyState');
                if (readyState !== 'complete') {
                    let display = window.getComputedStyle(t, null).display;
                    let e = t.getAttribute(onload);
                    if (display === 'block' && (!e || e.length <= 0)) {
                        d += "display: 'none' !important;";
                        t.onload = function() {
                            t.setAttribute('il_readyState', 'complete');
                            t.onload = null;
                            this.removeStyles(t);
                        }.bind(this);
                    }
                }
            }
            // if (this.nodeNameEqual(t, 'HTML')) {
            //     return;
            // }
            if (this.nodeNameEqual(t, 'HTML') || this.nodeNameEqual(t, 'BODY') || this.nodeNameEqual(t, 'IFRAME')) {
                d += 'background-color: ' + this.css.bodyColor + ' !important;' + 'background: ' + this.css.bodyColor + ' !important;';
                this.setStyle(t, d);
                return;
            }
            let s = window.getComputedStyle(t, null);
            d += this.borderColorProcess(t, s, '');
            d += this.borderColorProcess(t, s, '-top');
            d += this.borderColorProcess(t, s, '-left');
            d += this.borderColorProcess(t, s, '-right');
            d += this.borderColorProcess(t, s, '-bottom');
            let colorBrightnessP = this.darkMode?0.5:1;
            let addBrightness = false;
            let isBrightness = this.parantIsBrightness(t);
            if (s.background.match('url')) {
                addBrightness = true;
                const backgroundColor = s.backgroundColor;
                const backgroundColorRGBA = this.rgbaParser(backgroundColor);
                if (backgroundColorRGBA && backgroundColorRGBA[3] > 0) {
                    if (this.isGray(backgroundColorRGBA)) {
                        /* 已经设置变暗了，这里不能设置颜色，不然颜色会突出 */
                        d += 'background-color: ' + this.colorMake(0, 0, 0, 0) + ' !important;';
                    }
                }
            }
            if (!addBrightness && !isBrightness) {
                const backgroundImg = s.backgroundImage;
                if (backgroundImg && backgroundImg != 'none') {
                    /* 渐变处理 */
                    if (backgroundImg.match('gradient')) {
                        d += 'background-image: ' + this.colorsProcess(backgroundImg, colorBrightnessP) + ' !important;';
                    } else {
                        addBrightness = true;
                    }
                }
            }
            let isFullBackgroundNode = function (c) {
                let limitWidth = window.screen.width*0.7;
                let width = parseFloat(c.width);
                return (c.position !== 'fixed' && c.position !== 'relative') && width > limitWidth;
            };
            if (!addBrightness) {
                const backgroundRGBA = this.rgbaParser(s.background);
                if (backgroundRGBA && backgroundRGBA[3] > this.alphaLimit) {
                    let color = this.css.bgColor;
                    if (isFullBackgroundNode(s)) {
                        color = 'rgba(0, 0, 0, 0)';
                    } else if (!this.isGray(backgroundRGBA)) {
                        color = this.colorBrightness(backgroundRGBA, colorBrightnessP);
                    }
                    d += 'background: ' + color + ' !important;';
                }
            }
            if (!addBrightness) {
                const backgroundColor = s.backgroundColor;
                const backgroundColorRGBA = this.rgbaParser(backgroundColor);
                if (backgroundColorRGBA && backgroundColorRGBA[3] > this.alphaLimit) {
                    let color = this.css.bgColor;
                    if (isFullBackgroundNode(s)) {
                        color = 'rgba(0, 0, 0, 0)';
                    } else if (!this.isGray(backgroundColorRGBA)) {
                        color = this.colorBrightness(backgroundColorRGBA, colorBrightnessP);
                    }
                    d += 'background-color: ' + color + ' !important;';
                }
            }
            if (!addBrightness) {
                const shadowColor = s.boxShadow;
                const textShadow = s.textShadow;
                if (shadowColor && shadowColor !== 'none') {
                    d += 'box-shadow: ' + this.colorsProcess(shadowColor, colorBrightnessP) + ' !important;';
                }
                if (textShadow && textShadow !== 'none') {
                    d += 'text-shadow: ' + this.colorsProcess(textShadow, colorBrightnessP) + ' !important;';
                }
            }
            const color = s.color;
            const colorRGBA = this.rgbaParser(color);
            const colorBrightness = this.transparentMode?1.15:1;
            if (colorRGBA && colorRGBA[3] > 0) {
                const colorGray = this.isGray(colorRGBA);
                if (this.nodeNameEqual(t, 'A') || this.nodeNameEqual(t, 'INPUT') || this.nodeNameEqual(t, 'SELECT')) {
                    if (colorGray) {
                        let textColor = this.css.linkTextColor;
                        this.nodeNameEqual(t, 'INPUT') && (textColor = this.css.inputTextColor);
                        this.nodeNameEqual(t, 'SELECT') && (textColor = this.css.selectTextColor);
                        d += 'color: ' + textColor + ' !important;'
                    } else {
                        d += 'color: ' + this.colorMake(colorRGBA[0]*colorBrightness, colorRGBA[1]*colorBrightness, colorRGBA[2]*colorBrightness, colorBrightnessP) + ' !important;'
                    }
                } else {
                    if (colorGray) {
                        d += 'color: ' + this.css.textColor + ' !important;'
                    } else {
                        d += 'color: ' + this.colorMake(colorRGBA[0]*colorBrightness, colorRGBA[1]*colorBrightness, colorRGBA[2]*colorBrightness, colorBrightnessP) + ' !important;'
                    }
                }
            }
            (this.nodeNameEqual(t, 'IMG') || this.nodeNameEqual(t, 'SVG')) && (addBrightness = true);
            addBrightness && !isBrightness &&  (d += "filter: brightness(" + this.css.brightness + ") !important;");
            this.setStyle(t, d);
            if (!addBrightness || isBrightness) {
                this.addStyleElementIfNeed();
                this.processPseudo(t, '::before', this.darkMode?'il_dark_brightness_before':'il_trans_brightness_before');
                this.processPseudo(t, '::after', this.darkMode?"il_dark_brightness_after":'il_trans_brightness_after');
            }
        };
        ILTheme.prototype.colorsProcess = function (c, p) {
            let newC = c;
            let colors = c.match(/rgba?\(.+?\)/g);
            for (let color of colors) {
                let rgba = this.rgbaParser(color);
                if (rgba) {
                    let bgColorRGBA = this.rgbaParser(this.css.bgColor);
                    let alpha = rgba[3];
                    if (!this.darkMode && alpha > 0.8) {
                        alpha = 0.4;
                    }
                    let bgColor = this.colorMake(bgColorRGBA[0], bgColorRGBA[1], bgColorRGBA[2], alpha);
                    if (!this.isGray(rgba)) {
                        bgColor = this.colorBrightness(rgba, p);
                    }
                    newC = newC.replace(color, bgColor);
                }
            }
            return newC;
        };
        ILTheme.prototype.nodeNameEqual = function (t, r) {
            return t.nodeName.toLowerCase() === r.toLowerCase();
        };
        ILTheme.prototype.parantIsBrightness = function (t) {
            let exist = false;
            let parentNode = t.parentNode;
            while (!exist && parentNode && !this.nodeNameEqual(parentNode, 'HTML')) {
                s = parentNode.getAttribute('style');
                exist = (s !== null && s.match('brightness') !== null);
                parentNode = parentNode.parentNode;
            }
            return exist;
        };
        ILTheme.prototype.borderColorProcess = function (t, s, c) {
            let d = '';
            const borderWidth = s.getPropertyValue('border' + c + '-width');
            if (borderWidth && borderWidth !== 'none' && !borderWidth.match('0px')) {
                const borderColor = s.getPropertyValue('border' + c + '-color');
                const rgba = this.rgbaParser(borderColor);
                if (rgba && rgba[3] > 0) {
                    let color = this.colorMake(rgba[0], rgba[1], rgba[2], 0.1);
                    d += 'border' + c + '-color: ' + color + ' !important;';
                }
            }
            return d;
        };
        ILTheme.prototype.processPseudo = function (t, p, c) {
            let s = window.getComputedStyle(t, p);
            let content = s.content;
            if (!content || content === 'none') {
                return;
            }
            let addBrightness = false;
            const background = s.background;
            const backgroundRGBA = this.rgbaParser(background);
            if (background.match('url') || (backgroundRGBA && backgroundRGBA[3] > 0)) {
                addBrightness = true;
            }
            if (!addBrightness) {
                const backgroundImg = s.backgroundImage;
                if (backgroundImg && backgroundImg !== "none") {
                    addBrightness = true;
                }
            }
            if (!addBrightness) {
                const backgroundColor = s.backgroundColor;
                const backgroundColorRGBA = this.rgbaParser(backgroundColor);
                if (backgroundColorRGBA && backgroundColorRGBA[3] > 0) {
                    addBrightness = true;
                }
            }
            if (!addBrightness) {
                const color = s.color;
                const colorRGBA = this.rgbaParser(color);
                if (colorRGBA && colorRGBA[3] > 0) {
                    addBrightness = true;
                }
            }
            !addBrightness && (addBrightness = this.borderColorProcess(t, s, "").length > 0);
            !addBrightness && (addBrightness = this.borderColorProcess(t, s, "-top").length > 0);
            !addBrightness && (addBrightness = this.borderColorProcess(t, s, "-left").length > 0);
            !addBrightness && (addBrightness = this.borderColorProcess(t, s, "-right").length > 0);
            !addBrightness && (addBrightness = this.borderColorProcess(t, s, "-bottom").length > 0);
            addBrightness && t.classList.add(c);
        };
        ILTheme.prototype.colorBrightness = function (rgba, p) {
            if (!rgba) return 'rgba(0, 0, 0, 0)';
            const r = parseInt(rgba[0]*p);
            const g = parseInt(rgba[1]*p);
            const b = parseInt(rgba[2]*p);
            const a = rgba[3];
            r>255?255:r;
            g>255?255:g;
            b>255?255:b;
            a>1?1:a;
            return this.colorMake(r, g, b, a);
        };
        ILTheme.prototype.colorMake = function (r, g, b, a = 1) {
            if (r > 255) r = 255;
            if (g > 255) g = 255;
            if (b > 255) b = 255;
            if (a > 1) a = 1;
            if (a === 1) {
                return 'rgb(' + r + ', ' + g + ', ' + b + ')';
            } else {
                return 'rgba(' + r + ', ' + g + ', ' + b + ', ' + a + ')';
            }
        };
        ILTheme.prototype.isGray = function (rgb) {
            if (!rgb) return false;
            const limit = 20;
            const r = parseInt(rgb[0]);
            const g = parseInt(rgb[1]);
            const b = parseInt(rgb[2]);
            const i = (r + g + b)/3;
            return Math.abs(i - r) < limit && Math.abs(i - g) < limit && Math.abs(i - b) < limit;
        };
        ILTheme.prototype.rgbaParser = function (color) {
            if (!color) {
                return [0, 0, 0, 0];
            }
            let orig = color;
            let retColor = null;
            let rgba = orig.replace(/\s/g, '').match(/^rgba?\((\d+),(\d+),(\d+),?([^,\s)]+)?/i);
            if (rgba) {
                retColor = [];
                retColor[0] = (rgba && rgba[1] || "0").trim();
                retColor[1] = (rgba && rgba[2] || "0").trim();
                retColor[2] = (rgba && rgba[3] || "0").trim();
                retColor[3] = (rgba && rgba[4] || "1").trim();
            }
            return retColor;
        };
        ILTheme.prototype.setStyle = function (t, b) {
            const c = t.getAttribute("style") || "";
            const n = c + (c.slice(-1)!==';'?';':'') + b;
            t.setAttribute("style", n);
            t.setAttribute("il_style", b);
        };
        ILTheme.prototype.addStyleElementIfNeed = function() {
            if (!document.getElementById("il_css")) {
                let n = document.createElement("style");
                n.setAttribute("id", "il_css");
                let innerHTML = (".il_dark_brightness_before::before { filter: brightness(50%) !important; color: {il_dark_brightness_before_color} !important; }" +
                    ".il_dark_brightness_after::after { filter: brightness(50%) !important; color: {il_dark_brightness_after_color} !important; }" +
                    ".il_trans_brightness_before::before { filter: brightness(100%) !important; color: {il_trans_brightness_before_color} !important; }" +
                    ".il_trans_brightness_after::after { filter: brightness(100%) !important; color: {il_trans_brightness_after_color} !important; }");
                innerHTML = innerHTML.replace('{il_dark_brightness_before_color}', this.css.textColor);
                innerHTML = innerHTML.replace('{il_dark_brightness_after_color}', this.css.textColor);
                innerHTML = innerHTML.replace('{il_trans_brightness_before_color}', this.css.textColor);
                innerHTML = innerHTML.replace('{il_trans_brightness_after_color}', this.css.textColor);
                n.innerHTML = innerHTML;
                if (document !== null && document.body !== null) {
                    document.body.appendChild(n);
                }
            }
        };
        ILTheme.prototype.removeStyleElement = function () {
            let c = document.getElementById("il_dark_css");
            c && c.remove();
        };
        ILTheme.prototype.removeInjectStyles = function () {
            this.removeStyleElement();
            let root = this.targetNode;
            let elements = root.getElementsByTagName("*");
            this.removeStyles(root);
            for (let b = 0; b < elements.length; b++) {
                this.removeStyles(elements[b]);
            }
        };
        ILTheme.prototype.removeStyles = function (t) {
            let s = t.getAttribute("style");
            let sl = t.getAttribute("il_style");
            if (s && sl) {
                let sp = s.split(';');
                let slp = sl.split(';');
                if (slp.length > 0 && sp.length > 0) {
                    let newSp = [];
                    for (let tsp of sp) {
                        let contains = false;
                        for (let tlp of slp) {
                            let tlp_ = tlp.replace(/(^\s*)|(\s*$)/g, "");
                            let tsp_ = tsp.replace(/(^\s*)|(\s*$)/g, "");
                            if (tsp_ === tlp_) {
                                contains = true;
                                break;
                            }
                            // 颜色alpha存在浮点问题
                            let tsp_important = tsp_.match('!important');
                            let tlp_important = tlp_.match('!important');
                            if (tsp_important && tlp_important && tsp_.match('rgba') && tlp_.match('rgba')) {
                                let t1 = tsp_.split(':');
                                let t2 = tlp_.split(':');
                                if (t1.length >= 2 && t2.length >= 2) {
                                    let t1Key = t1[0];
                                    let t1Value = t1[1];
                                    let t2Key = t2[0];
                                    let t2Value = t2[1];
                                    if (t1Key === t2Key) {
                                        const color1 = this.rgbaParser(t1Value);
                                        const color2 = this.rgbaParser(t2Value);
                                        if (color1 && color2 && color1[0] === color2[0] && color1[1] === color2[1] && color1[2] === color2[2]) {
                                            if (Math.abs(color1[3] - color2[3]) < 0.02) {
                                                contains = true;
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (!contains && tsp.length > 0) {
                            newSp.push(tsp);
                        }
                    }
                    t.setAttribute('style', newSp.join(';'));
                }
            }
            s = t.getAttribute("style");
            if (!s || s.length <= 0) {
                t.removeAttribute("style");
            }
            t.removeAttribute("il_style");
            this.removePseudoStyles(t);
        };
        ILTheme.prototype.removePseudoStyles = function (t) {
            t.classList.remove('il_dark_brightness_before');
            t.classList.remove('il_trans_brightness_before');
            t.classList.remove('il_dark_brightness_after');
            t.classList.remove('il_trans_brightness_after');
        };
        ILTheme.prototype.isElementInViewport = function (t) {
            var rect = t.getBoundingClientRect();
            return (
                rect.x < (window.innerWidth || document.documentElement.clientWidth) &&
                rect.y < (window.innerHeight || document.documentElement.clientHeight) &&
                ((rect.x + rect.width) > 0 || (rect.y + rect.height) > 0)
            );
        };
        ILTheme.prototype.listenCallback = function (mutations, observer) {
            let elements = new Array();
            for(let mutation of mutations) {
                if (mutation.type == 'childList') {
                    for (let n of mutation.addedNodes) {
                        if (n.nodeType === Node.ELEMENT_NODE) {
                            if (this.nodeNameEqual(n, 'LINK') && n.rel === 'stylesheet') {
                                n.addEventListener('load', function() {
                                    ILThemeInstance.removeInjectStyles();
                                    ILThemeInstance.injectStyles();
                                }, false);
                            }
                            if (!this.nodeNameEqual(n, 'META') &&
                                !this.nodeNameEqual(n, 'LINK') &&
                                !this.nodeNameEqual(n, 'SCRIPT') &&
                                !this.nodeNameEqual(n, 'svg') &&
                                !this.nodeNameEqual(n, 'g') &&
                                !this.nodeNameEqual(n, 'path') &&
                                !elements.includes(n)) {
                                    elements.push(n);
                                    let c = n.getElementsByTagName("*");
                                    for (let t of c) {
                                        elements.push(t);
                                    }
                            }
                        }
                    }
                } else if (mutation.type == 'attributes') {
                    let n = mutation.target;
                    if (!elements.includes(n)) {
                        elements.push(n);
                        if (mutation.attributeName === "class") {
                            let c = n.getElementsByTagName("*");
                            for (let t of c) {
                                elements.push(t);
                            }
                        }
                    }
                }
            }
            elements = Array.from(new Set(elements));
            let elementsInView = new Array();
            let delayElements = new Array();
            for (let t of elements) {
                if (!this.delayMode || this.isElementInViewport(t)) {
                    elementsInView.push(t);
                } else {
                    delayElements.push(t);
                }
            }
            let len = parseInt(delayElements.length/this.delayEcho);
            len = len<=0?1:len;
            let interval = 0;
            for(let i = 0; i < delayElements.length; i += len) {
                let subDelayElements = delayElements.slice(i, i+len)
                setTimeout(function() {
                    this.listenStop(observer);
                    for (let t of subDelayElements) {
                        this.removeStyles(t);
                        this.processStyles(t);
                    }
                    this.listenStart(observer);
                }.bind(this), interval);
                interval+=this.delayExpandTime;
            }

            this.listenStop(observer);
            for (let t of elementsInView) {
                this.removeStyles(t);
                this.processStyles(t);
            }
            this.listenStart(observer);
        };
        ILTheme.prototype.listenStop = function (observer) {
            var mutations = observer.takeRecords();
            if (mutations.length > 0) {
                this.listenCallback(mutations, observer);
            }
            observer.disconnect();
        };
        ILTheme.prototype.listenStart = function (observer) {
            observer.observe(this.targetNode, this.config);
        };
        ILTheme.prototype.listenElement = function () {
            if (!this.observer) {
                this.observer = new MutationObserver(this.listenCallback.bind(this));
            }
            this.listenStart(this.observer);
        };
        ILTheme.prototype.listen = function () {
            this.listenElement();
        };
        ILTheme.prototype.disconnect = function () {
            if (this.observer) {
                this.observer.disconnect();
            }
        };
        return ILTheme;
    }());
    var ILThemeInstance = new ILTheme();
    var ILThemeConfig = /** @class */ (function () {
        function ILThemeConfig() {
            this.dark_theme_css = {
                brightness: '70%',
                mainColor: 'rgb(35, 34, 38)',
                bodyColor: 'rgb(35, 34, 38)',
                bgColor: 'rgb(35, 34, 38)',
                linkTextColor: 'rgb(38, 70, 175)',
                textColor: 'rgb(133, 132, 135)',
                inputTextColor: 'rgb(133, 132, 135)',
                selectTextColor: 'rgb(133, 132, 135)'
            };
            this.transparent_theme_css = {
                brightness: '100%',
                mainColor: 'rgba(255, 255, 255, 0.4)',
                bodyColor: 'rgba(0, 0, 0, 0)',
                bgColor: 'rgba(0, 0, 0, 0.3)',
                linkTextColor: 'rgb(67, 160, 215)',
                textColor: 'rgb(238, 238, 238)',
                inputTextColor: 'rgb(238, 238, 238)',
                selectTextColor: 'rgb(238, 238, 238)'
            };
            this.black_theme_css = {
                brightness: '100%',
                mainColor: 'rgb(0, 0, 0)',
                bodyColor: 'rgb(0, 0, 0)',
                bgColor: 'rgb(29, 29, 29)',
                linkTextColor: 'rgb(0, 141, 255)',
                textColor: 'rgb(255, 255, 255)',
                inputTextColor: 'rgb(238, 238, 238)',
                selectTextColor: 'rgb(238, 238, 238)'
            };
            this.style = 'common';
            this.loopTime = null;
        }
        ILThemeConfig.prototype.getTheme = function () {
            return this.style;
        };
        ILThemeConfig.prototype.chnageTheme = function (style) {
            this.style = style;
            ILThemeInstance.style = this.style;
            this.updateTheme();
        };
        ILThemeConfig.prototype.updateTheme = function () {
            ILThemeInstance.darkMode = false;
            ILThemeInstance.transparentMode = false;
            theme_css = {};
            if (this.style === 'dark') {
                ILThemeInstance.darkMode = true;
                theme_css = this.dark_theme_css;
            } else if (this.style === 'transparent') {
                ILThemeInstance.transparentMode = true;
                theme_css = this.transparent_theme_css;
            } else if (this.style === 'black') {
                theme_css = this.black_theme_css;
            }
            ILThemeInstance.css = theme_css;
            ILThemeInstance.disconnect();
            ILThemeInstance.removeInjectStyles();
            if (Object.keys(theme_css).length > 0) {
                ILThemeInstance.injectStyles();
                ILThemeInstance.listen();
            }
        };
        ILThemeConfig.prototype.init = function () {
            window.ILTheme = ILThemeInstance;
            this.updateTheme();
        };
        return ILThemeConfig;
    }());
    window.ILThemeConfig = new ILThemeConfig();
    window.ILThemeConfig.init();
})(window);
// window.ILThemeConfig.chnageTheme("transparent");
